#include "Vector.h"

#include "Point.h"

#include <math.h>

Vector::Vector() : x(Element[0]), y(Element[1]), z(Element[2])
{
}

Vector::Vector(const int i) : x(Element[0]), y(Element[1]), z(Element[2])
{
	const float s = (float)i;

	x = s;
	y = s;
	z = s;
}

Vector::Vector(const Vector &v) : x(Element[0]), y(Element[1]), z(Element[2])
{
	x = v.x;
	y = v.y;
	z = v.z;
}

Vector::Vector(const Point &P) : x(Element[0]), y(Element[1]), z(Element[2])
{
	x = P.x;
	y = P.y;
	z = P.z;
}

Vector::Vector(float v_x, float v_y, float v_z) : x(Element[0]), y(Element[1]), z(Element[2])
{
	x = v_x;
	y = v_y;
	z = v_z;
}

Vector &Vector::operator=(const Vector &v)
{
	x = v.x;
	y = v.y;
	z = v.z;

	return *this;
}

Vector Vector::operator+() const
{
	return *this;
}

Vector Vector::operator-() const
{
	return Vector(-x, -y, -z);
}

Vector &Vector::operator+=(const Vector &v)
{
	x += v.x;
	y += v.y;
	z += v.z;

	return *this;
}

Vector &Vector::operator-=(const Vector &v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;

	return *this;
}

Vector &Vector::operator*=(float s)
{
	x *= s;
	y *= s;
	z *= s;

	return *this;
}

Vector &Vector::operator/=(float s)
{
	float r = 1.0f / s;

	return *this *= r;
}

bool operator==(const Vector &U, const Vector &v)
{
	if(U.x == v.x && U.y == v.y && U.z == v.z)
		return true;
	else
		return false;
}

bool operator!=(const Vector &U, const Vector &v)
{
	if(U.x != v.x || U.y != v.y || U.z != v.z)
		return true;
	else
		return false;
}

bool operator>(const Vector &u, const Vector &v)
{
	if((u^2) > (v^2))
		return true;
	else
		return false;
}

bool operator<(const Vector &u, const Vector &v)
{
	if((u^2) < (v^2))
		return true;
	else
		return false;
}

Vector operator+(const Vector &u, const Vector &v)
{
	return Vector(u.x + v.x, u.y + v.y, u.z + v.z);
}

Vector operator-(const Vector &u, const Vector &v)
{
	return Vector(u.x - v.x, u.y - v.y, u.z - v.z);
}

float operator*(const Vector &u, const Vector &v)
{
	return u.x * v.x + u.y * v.y + u.z * v.z;
}

Vector operator*(float s, const Vector &v)
{
	return Vector(s * v.x, s * v.y, s * v.z);
}

Vector operator*(const Vector &v, float s)
{
	return Vector(v.x * s, v.y * s, v.z * s);
}

Vector operator/(const Vector &v, float s)
{
	float r = 1.0f / s;

	return Vector(v.x * r, v.y * r, v.z * r);
}

float operator^(const Vector &v, const int n)
{
	switch(n)
	{
		case 2: return v*v;
		default: return 1.0f;
	}
}

float operator^(const Vector &u, const Vector &v)
{
	return acosf((u / u.length()) * (v / v.length()));
}

Vector operator%(const Vector &u, const Vector &v)
{
	return Vector(u.y * v.z - u.z * v.y, u.z * v.x - u.x * v.z, u.x * v.y - u.y * v.x);
}

float Vector::length() const
{
	return sqrtf(x*x + y*y + z*z);
}

float N(const Vector &v)
{
	return sqrtf(v.x*v.x + v.y*v.y + v.z*v.z);
}

float N2(const Vector &v)
{
	return v.x*v.x + v.y*v.y + v.z*v.z;
}

Vector &Vector::normalize()
{
	return *this /= this->length();
}
